#!/bin/bash
set -e

immutable_adapter_cmd="deepin-immutable-ctl admin exec --"
cmd_args=("$@")
cmd_name=$(basename "$0")

take_over_commands=("apt" "apt-get" "aptitude" "dpkg")

# Function to check if the system is booted in immutable mode
# Returns 1 if in immutable mode, 0 otherwise
function in_immutable_mode {
    command -v deepin-immutable-ctl &> /dev/null && \
    [[ $(deepin-immutable-ctl --immutable-status) =~ "Immutable mode:true" ]]
}

# Indicates that the writable function has been activated
function writable_activated {
    [[ -f "/run/deepin-immutable-writable/booted" ]]
}

function exec_original {
    exec -a "${cmd_name}" "${cmd_name}.real" "${cmd_args[@]}"
}

function exec_immutable {
    IFS=' ' read -ra immutable_cmd_parts <<< "${immutable_adapter_cmd}"
    new_cmd_name="${immutable_cmd_parts[0]}"
    new_cmd_args=("${immutable_cmd_parts[@]:1}" "${cmd_name}.real" "${cmd_args[@]}")
    exec -a "${cmd_name}" "${new_cmd_name}" "${new_cmd_args[@]}"
}

function main {
    if writable_activated || ! in_immutable_mode || [ "${RUN_BY_IMMUTABLE_CTL:-}" == "1" ] || [ "$(id -u)" -ne 0 ]; then
        exec_original
    fi

    for cmd in "${take_over_commands[@]}"; do
        if [[ "${cmd_name}" == "${cmd}" ]]; then
            exec_immutable
        fi
    done
    exec_original
}

main "$@"