#!/bin/bash

# This script changes the ownership of user home directories to match their corresponding UID and GID.
# It processes each home directory under /home and updates the ownership if the UID is greater than or equal to 1000.

# Function to get the UID and GID corresponding to a user's home directory
get_home_uid_gid() {
    local home_dir="$1"
    local uid=0
    local gid=0

    # Use getent to retrieve user information, ensuring we match the home directory exactly
    IFS=: read -r user passwd uid gid info home shell < <(getent passwd | awk -F: -v dir="$home_dir" '$6 == dir')

    if [[ "$home" == "$home_dir" ]]; then
        echo "$uid $gid"
        return 0
    fi

    echo "0 0"
    return 1
}

# Function to recursively change the owner of a directory and its contents
chown_home_dir() {
    local home_dir="$1"
    local target_uid="$2"
    local target_gid="$3"

    # Get the current UID and GID of the directory
    local current_uid=$(stat -c '%u' "$home_dir")
    local current_gid=$(stat -c '%g' "$home_dir")

    # Check if a change is needed
    if [[ "$current_uid" != "$target_uid" || "$current_gid" != "$target_gid" ]]; then
        echo "Changing ownership of $home_dir from UID $current_uid:GID $current_gid to UID $target_uid:GID $target_gid"
        chown -R "$target_uid:$target_gid" "$home_dir"
    else
        echo "No need to change ownership for $home_dir (UID $current_uid:GID $current_gid)"
    fi
}

# Main function to process valid home directories
chown_valid_home_dirs() {
    local root_dir="/"

    # Iterate through all subdirectories in /home
    for home_dir in /home/*; do
        if [[ -d "$home_dir" ]]; then
            echo "Processing home directory: $home_dir"

            # Get the UID and GID corresponding to the home directory
            read -r uid gid < <(get_home_uid_gid "$home_dir")
            if [[ $uid -ge 1000 ]]; then
                # If UID >= 1000, change the ownership of the home directory
                chown_home_dir "$home_dir" "$uid" "$gid"
            else
                echo "Skipping $home_dir: UID is less than 1000"
            fi
        fi
    done
}

# Call the main function
chown_valid_home_dirs
